package com.abaffydesign;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

public class DatabaseHelper extends SQLiteOpenHelper {

	static final String dbName = "WifiLocator";
	static final String measurementsTable = "measurements";
	static final String colID = "MeasurementID";
	static final String colBSSID = "BSSID";
	static final String colSSID = "SSID";
	static final String colStrength = "Strength";
	static final String colPositionX = "PositionX";
	static final String colPositionY = "PositionY";
	
	public DatabaseHelper(Context context) {
		super(context, dbName, null, 36);
	}

	@Override
	public void onCreate(SQLiteDatabase db) {

		  db.execSQL("CREATE TABLE "+measurementsTable+" ("+
				  colID+" INTEGER PRIMARY KEY AUTOINCREMENT, "+
				  colBSSID+" TEXT , "+
				  colSSID+" TEXT , "+
				  colStrength+ " INTEGER , "+
				  colPositionX+ " INTEGER , "+
				  colPositionY+ " INTEGER)"); 
	 }

	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
		db.execSQL("DROP TABLE IF EXISTS " + measurementsTable);
		onCreate(db);
	}
	
	public Cursor getAllMeasurements() {
		SQLiteDatabase db = this.getReadableDatabase();
		Cursor cur = db.rawQuery("SELECT 	" + colID + " as _id, " + colBSSID
				+ ", " + colSSID + ", " + colStrength + ", " + colPositionX
				+ ", " + colPositionY + " from " + measurementsTable,
				new String[] {});

		return cur;
	}
	
	public void resetData() {
		SQLiteDatabase db = this.getReadableDatabase();
		db.execSQL("DELETE FROM " + measurementsTable + " WHERE 1");
	}
	
	public void insertMeasurement(String BSSID, String SSID, int strength, int posx, int posy) {
		SQLiteDatabase db = this.getWritableDatabase();
		ContentValues cv = new ContentValues();
		
		cv.put(colBSSID, BSSID);
		cv.put(colSSID, SSID);
		cv.put(colStrength, strength);
		cv.put(colPositionX, posx);
		cv.put(colPositionY, posy);
		
		db.insert(measurementsTable, colID, cv);
		db.close();
	}
}
