package com.abaffydesign;

import java.util.ArrayList;

import com.abaffydesign.R;

import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.IntentFilter;
import android.graphics.PointF;
import android.net.wifi.WifiManager;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.content.Intent;

public class MapActivity extends Activity {
	
    private WifiMapView mMapView;
    private WifiLocator mWifiLocator;

	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
        setContentView(R.layout.main);
        
        mMapView = (WifiMapView) findViewById(R.id.mapView);
        mWifiLocator = new WifiLocator(this);
        
        IntentFilter i = new IntentFilter(WifiManager.SCAN_RESULTS_AVAILABLE_ACTION);
        registerReceiver(new BroadcastReceiver() {
        	public void onReceive(Context c, Intent i) { 
        		if(mWifiLocator.getScanAction() == WifiLocator.MAP_SCAN_ACTION_SAVE) {
        			 
        			mWifiLocator.saveCurrentPosition();
        			mMapView.setMeasurementPoints(mWifiLocator.getMeasurements());
        			mMapView.invalidate();
        			mWifiLocator.setScanAction(-1);
        		}
        		else if(mWifiLocator.getScanAction() == WifiLocator.MAP_SCAN_ACTION_LOCATE) {
        			
        			//mWifiLocator.getCurrentPosition();
        			Log.i("AppMessages", "Scan for locate done...");
        			 
        			WifiPointF location = mWifiLocator.getCurrentPosition();
        			
        			mMapView.setMeasurementPoints(mWifiLocator.getInterpolatedMeasurements());
        			mMapView.setDelaunayTriangulation(mWifiLocator.getDelaunayTriangulation());
        			mMapView.invalidate();
        	    	
        	    	if(location == null) {
        	    		/*AlertDialog alertDialog = new AlertDialog.Builder(this).create();
        	    		alertDialog.setTitle("No data found");
        	    		alertDialog.setMessage("Unable to calculate position from 0 points");
        	    		alertDialog.setButton("OK", new DialogInterface.OnClickListener() {
        	    		   public void onClick(DialogInterface dialog, int which) {
        	    		      // here you can add functions
        	    		   }
        	    		});
        	    		alertDialog.setIcon(R.drawable.menu_compass);
        	    		alertDialog.show();*/
        	    		return;
        	    	}
        	    	
        	    	mMapView.setLocation(location);
        	    	mWifiLocator.setScanAction(-1);
        		}
        	} 
        }, i); 
    }
    
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater menuInflater = getMenuInflater();
		menuInflater.inflate(R.layout.map_menu, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {

		switch (item.getItemId()) {
		case R.id.menu_save:
			saveCurrentPosition();
			return true;
			
		case R.id.menu_mapmode:
			switchToInterlacedMap();
			return true;
			
		case R.id.menu_detect:
			locate();
			return true;
			
		case R.id.menu_reset:
			resetMapData();
			return true;
			
		case R.id.menu_zoomin:
			mMapView.zoomIn();
			return true;
			
		case R.id.menu_zoomout:
			mMapView.zoomOut();
			return true;
			
		case R.id.menu_setbestpoint:
			mWifiLocator.setLocateAction(WifiLocator.MAP_LOCATE_ACTION_BESTPOINT);
			return true;
			
		case R.id.menu_setinterpolation:
			mWifiLocator.setLocateAction(WifiLocator.MAP_LOCATE_ACTION_INTERPOLATION);
			return true;
			
		case R.id.menu_switchinterpolationpoints:
			mMapView.switchPointMode();
			return true;
			
		case R.id.menu_switchheatmaps:
			mMapView.switchHeatmapMode();
			return true;

		default:
			return super.onOptionsItemSelected(item);
		}
	}
    
    
    public void switchToInterlacedMap() {
        Log.i("AppMessages", "Switching to interlaced map view.");
        
        ArrayList<Measurement> measurements = mWifiLocator.getMeasurements();
        
        for(Measurement m : measurements) {
        	Log.i("WifiScanDB", m.SSID+" - "+m.signalStrength+" dB at "+m.positionX+","+m.positionY);
        }
        
        
        
        mMapView.setMeasurementPoints(measurements);
        mMapView.switchMapMode();
    }
    
    public void resetMapData() {
    	
    	mWifiLocator.resetData();
    	mMapView.setMeasurementPoints(mWifiLocator.getMeasurements());
    	mMapView.invalidate();
    }
    
    public void saveCurrentPosition() {
    	PointF pos = mMapView.getMapPosition();
    	
    	Log.i("AppMessages", "Starting save scan... Current position: "+pos.x+","+pos.y);
    	
    	mWifiLocator.setScanAction(WifiLocator.MAP_SCAN_ACTION_SAVE);
    	mWifiLocator.startScanOnPosition(Math.round(pos.x), Math.round(pos.y));
    }
    
    public void locate() {
    	PointF pos = mMapView.getMapPosition();
    	
    	Log.i("AppMessages", "Starting locate scan... Current position: "+pos.x+","+pos.y);
    	
    	mWifiLocator.setScanAction(WifiLocator.MAP_SCAN_ACTION_LOCATE);
    	mWifiLocator.startScanOnPosition(Math.round(pos.x), Math.round(pos.y));
    }
    
    @Override
    protected void onDestroy() {
    	mWifiLocator.closeDB();
    	super.onDestroy();
    }
}