package com.abaffydesign;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.TreeMap;

import delaunay_triangulation.Delaunay_Triangulation;
import delaunay_triangulation.Point_dt;
import delaunay_triangulation.Triangle_dt;

import android.content.Context;
import android.database.Cursor;
import android.net.wifi.ScanResult;
import android.net.wifi.WifiManager;
import android.util.Log;

public class WifiLocator {
	
	private Context mContext;
	private DatabaseHelper mDatabase;
	private int mX, mY;
	private int mScanAction = -1;
	private int mLocateAction = 0;
	private ArrayList<Measurement> mInterpolatedMeasurements;
	private TreeMap<String, MeasurementPoint> mInterpolatedMeasurementPoints;
	private Delaunay_Triangulation mDelaunayTriangulation;
	
	public static final int MAP_SCAN_ACTION_LOCATE = 0;
	public static final int MAP_SCAN_ACTION_SAVE = 1;
	
	public static final int MAP_LOCATE_ACTION_BESTPOINT = 0;
	public static final int MAP_LOCATE_ACTION_INTERPOLATION = 1;

	public int getScanAction() {
		return mScanAction;
	}

	public void setScanAction(int scanAction) {
		this.mScanAction = scanAction;
	}

	public int getLocateAction() {
		return mLocateAction;
	}
	
	public void setLocateAction(int locateAction) {
		this.mLocateAction = locateAction;
	}
	
	public WifiPointF getCurrentPosition() {
		if(mLocateAction == 0)
			return detectLocationNearestSimilarPoint();
		
		return detectLocationInterpolatedMap();
	}
	
	/**
	 * Obsah
	 */
	double area(Point_dt a, Point_dt b, Point_dt c)
	{
		/*return 0.5 * (a.x() * (b.y() - c.y()) +
                b.getX() * (c.y() - a.y()) +
                c.getX() * (a.y() - b.y()));*/
		
		return ((a.x()*(b.y()-c.y())+b.x()*(c.y()-a.y())+c.x()*(a.y()-b.y()))/2);
	}
	
	/**
	 * Funkcia vzdy vyinterpoluje tazisko trojuholnika
	 * 
	 */
	public void interpolateCenter(Point_dt a, Point_dt b, Point_dt c) {
		
		double px = (a.x()+b.x()+c.x())/3;
		double py = (a.y()+b.y()+c.y())/3;
		
		Point_dt p = new Point_dt(px, py);
		
		// Vypocitame plochy
		double abc = area(a, b, c);
		double abp = area(a, b, p);
		double apc = area(a, p, c);
		double pbc = area(p, b, c);
		//System.out.println("Areas: ("+abp+","+apc+","+pbc+" |= "+abc+")");
		
		// Vypocitame vahy
		double u = pbc / abc;
		double v = apc / abc;
		double w = abp / abc;
		//System.out.println("Weights: ("+u+","+v+","+w+" |= "+(u+v+w)+")");
		
		MeasurementPoint point = new MeasurementPoint();
		
		MeasurementPoint atmp = mInterpolatedMeasurementPoints.get((int)a.x()+"x"+(int)a.y());
		MeasurementPoint btmp = mInterpolatedMeasurementPoints.get((int)b.x()+"x"+(int)b.y());
		MeasurementPoint ctmp = mInterpolatedMeasurementPoints.get((int)c.x()+"x"+(int)c.y());
		
		MeasurementPoint amp = preparePointForInterpolation(atmp, btmp, ctmp);
		MeasurementPoint bmp = preparePointForInterpolation(btmp, atmp, ctmp);
		MeasurementPoint cmp = preparePointForInterpolation(ctmp, atmp, btmp);
		
		// Kontrola spravnosti: vsetky body by mali mat rovnaky pocet APciek
		if(!(amp.measurements.size()==bmp.measurements.size() && amp.measurements.size()==cmp.measurements.size())) {
			System.out.println("Oops");
			return;
		}
		
		// Samotna interpolacia hodnoty pomocou barycentrickych vah
		
		for(Measurement m : amp.measurements) {
			
			int aStrength = m.signalStrength;
			int bStrength = 999;
			int cStrength = 999;
			
			for(Measurement tmp : bmp.measurements) {
				if(tmp.BSSID.equals(m.BSSID)) {
					bStrength = tmp.signalStrength;
					break;
				}
			}
			
			for(Measurement tmp : cmp.measurements) {
				if(tmp.BSSID.equals(m.BSSID)) {
					cStrength = tmp.signalStrength;
					break;
				}
			}

			int signalStrength = (int)((u*aStrength+v*bStrength+w*cStrength)/(u+v+w));
			Measurement interpolatedMeasurement = new Measurement(m.BSSID, m.SSID, signalStrength, (int)px, (int)py, true);
			
			point.measurements.add(interpolatedMeasurement);
			mInterpolatedMeasurements.add(interpolatedMeasurement);
			//System.out.println("Interpolated strength: "+signalStrength+" ("+aStrength+", "+bStrength+", "+cStrength+")");
		}
		
		// Pridame bod do mnoziny merani
		if(!mInterpolatedMeasurementPoints.containsKey((int)px+"x"+(int)py)) {
			point.posx = (int)px;
			point.posy = (int)py;
			
			mInterpolatedMeasurementPoints.put((int)px+"x"+(int)py, point);
		}
		
		
		if(abp > 500)
			interpolateCenter(a, b, p);
		if(apc > 500)
			interpolateCenter(a, p, c);
		if(pbc > 500)
			interpolateCenter(p, b, c);
		
	}
	
	public WifiPointF detectLocationInterpolatedMap() {
		
		ArrayList<Measurement> measurements = getMeasurements();
		
		if(measurements.size() == 0)
			return null;
		
		mInterpolatedMeasurementPoints = new TreeMap<String, MeasurementPoint>();
		
		mInterpolatedMeasurements = getMeasurements();
		
		
		if(mInterpolatedMeasurements.size() == 0)
			return null;
		
		
		WifiManager wm = (WifiManager) mContext.getSystemService(Context.WIFI_SERVICE);
		List<ScanResult> currentMeasurement = wm.getScanResults();
		mDelaunayTriangulation = new Delaunay_Triangulation();
		
		
		// Vytvorime grip pre delaunaya a internu mapu bodov
		for(Measurement m : measurements) {
			MeasurementPoint point;
			
			if(!mInterpolatedMeasurementPoints.containsKey(m.positionX+"x"+m.positionY)) {
				point = new MeasurementPoint();
				point.posx = m.positionX;
				point.posy = m.positionY;
				
				mInterpolatedMeasurementPoints.put(m.positionX+"x"+m.positionY, point);
				
				// delaunay
				Point_dt dpoint = new Point_dt(point.posx, point.posy);
				mDelaunayTriangulation.insertPoint(dpoint);
				System.out.println("adding point...");
			}
			else {
				point = mInterpolatedMeasurementPoints.get(m.positionX+"x"+m.positionY);
			}
			
			point.measurements.add(m);
		}
		
		// Vyinterpolujeme body barycentrickymi koordinatmi
		Triangle_dt curr = null;
		Iterator<Triangle_dt> it = mDelaunayTriangulation.trianglesIterator();
		while (it.hasNext()) {
			curr = it.next();
			
			if(curr.isHalfplane())
				continue;
			
			interpolateCenter(curr.p1(), curr.p2(), curr.p3());
			
		}
		
		System.out.println("END");
		
		
		MeasurementPoint bestPoint = null;
		float bestDeviation = 999999.0f;
		
		/*
		 * Preiterujeme vsetky body a pozrieme si zhodu, vyberieme si ten najlepsi,
		 * od ktoreho potom budeme robit interpolovane ciarocky
		 * 
		 * jednotlive wifi priemerujeme vazenym priemerom
		 */
		for(MeasurementPoint point : mInterpolatedMeasurementPoints.values()) {
			float deviation = calculateDeviationBetweenPoints(point, currentMeasurement);
			if(deviation < bestDeviation) {
				bestPoint = point;
				bestDeviation = deviation;
			}
			
			Log.i("wifilocator", "=== AVERAGE DEVIATION: "+deviation);
		}
		
		Log.i("Locator", "Best point position: "+bestPoint.posx+"x"+bestPoint.posy+", deviation: "+bestDeviation);
		
		return new WifiPointF(bestPoint.posx, bestPoint.posy, bestDeviation*10);
	}
	
	@SuppressWarnings("unused")
	public WifiPointF detectLocationNearestSimilarPoint() {
		
		ArrayList<Measurement> measurements = getMeasurements();
		
		if(measurements.size() == 0)
			return null;
		
		TreeMap<String, MeasurementPoint> measurementPoints = new TreeMap<String, MeasurementPoint>();
		
		WifiManager wm = (WifiManager) mContext.getSystemService(Context.WIFI_SERVICE);
		
		List<ScanResult> currentMeasurement = wm.getScanResults();
		
		for(Measurement m : measurements) {
			MeasurementPoint point;
			
			if(!measurementPoints.containsKey(m.positionX+"x"+m.positionY)) {
				point = new MeasurementPoint();
				point.posx = m.positionX;
				point.posy = m.positionY;
				
				measurementPoints.put(m.positionX+"x"+m.positionY, point);
			}
			else {
				point = measurementPoints.get(m.positionX+"x"+m.positionY);
			}
			
			point.measurements.add(m);
		}

		MeasurementPoint bestPoint = null;
		float bestDeviation = 999999.0f;
		
		/*
		 * Preiterujeme vsetky body a pozrieme si zhodu, vyberieme si ten najlepsi,
		 * od ktoreho potom budeme robit interpolovane ciarocky
		 * 
		 * jednotlive wifi priemerujeme vazenym priemerom
		 */
		for(MeasurementPoint point : measurementPoints.values()) {
			Log.i("wifilocator", point.posx+"x"+point.posy);
			
			float deviation = calculateDeviationBetweenPoints(point, currentMeasurement);
			if(deviation < bestDeviation) {
				bestPoint = point;
				bestDeviation = deviation;
			}
			
			Log.i("wifilocator", "=== AVERAGE DEVIATION: "+deviation);
		}
		
		Log.i("Locator", "Best point position: "+bestPoint.posx+"x"+bestPoint.posy);
		
		//if(bestPoint != null)
		//	return null;
		
		
		
		// nenasli sme nic lepsie ako ciste zlo
		if(bestPoint == null)
			return null;
		
		/*
		 * selectneme 4 body
		 * 
		 *    A          B
		 *    
		 *       tu som
		 *    
		 *    C          D
		 *    
		 * vyinterpolujeme ciary medzi mojou pozicou a kazdym bodom.
		 * nasledne sa pozrieme ci sa nemame po nejakej priamke posunut
		 */
		MeasurementPoint[] interpolArray = new MeasurementPoint[4];
		
		for(MeasurementPoint point : measurementPoints.values()) {
			Log.i("Debug", "===");
			
			if(point.posx < bestPoint.posx && point.posy > bestPoint.posy) { // vlavo hore
				
				Log.i("Debug", "vlavo hore");
				if(interpolArray[0] == null)
					interpolArray[0] = point;
				else if(distance(bestPoint, point) < distance(bestPoint, interpolArray[0]))
					interpolArray[0] = point;
					
			}
			else if(point.posx > bestPoint.posx && point.posy > bestPoint.posy) { // vpravo hore
				
				Log.i("Debug", "vpravo hore");
				if(interpolArray[1] == null)
					interpolArray[1] = point;
				else if(distance(bestPoint, point) < distance(bestPoint, interpolArray[1]))
					interpolArray[1] = point;
				
			}
			else if(point.posx < bestPoint.posx && point.posy < bestPoint.posy) { // vlavo dole
				
				Log.i("Debug", "vlavo dole");
				if(interpolArray[2] == null)
					interpolArray[2] = point;
				else if(distance(bestPoint, point) < distance(bestPoint, interpolArray[2]))
					interpolArray[2] = point;
				
			}
			else if(point.posx > bestPoint.posx && point.posy < bestPoint.posy){ // vpravo dole
				
				Log.i("Debug", "vpravo dole");
				if(interpolArray[3] == null)
					interpolArray[3] = point;
				else if(distance(bestPoint, point) < distance(bestPoint, interpolArray[3]))
					interpolArray[3] = point;
				
			}
		}
		
		for(int i = 0; i < interpolArray.length; i++) {
			MeasurementPoint p = interpolArray[i];
			if(p == null)
				continue;
			
			Log.i("Debug", "Point, posx:"+p.posy+" posxy: "+p.posx);
		}
		
		
		//ArrayList<MeasurementPoint> interpolatedPoints = new ArrayList<MeasurementPoint>();
		//interpolatedPoints.add(bestPoint);
		MeasurementPoint bestInterpolatedPoint = bestPoint;
		float bestInterpolatedDeviation = bestDeviation;
		
		/*
		 * Iteracia cez vsetky 'hranicne' body
		 */
		for(int i = 0; i < interpolArray.length; i++) {
			MeasurementPoint point = interpolArray[i];
			
			if(point == null)
				continue;
			
			if(point.posx == bestPoint.posx && point.posy == bestPoint.posy)
				continue;
			
			float distance = distance(point, bestPoint);
			
			int steps = Math.max((int)distance / 5, 20);
			
			/*
			 * Preinterpolujeme priamku medzi aktualnou poziciou a bestPointom
			 * a porovname zhodu bodov
			 */
			for(int j = 0; j < steps; j++) {
				
				float completion = (float)j / steps;
				Log.i("Debug", "Completion: "+completion);
				
				MeasurementPoint interpolatedPoint = new MeasurementPoint();
				interpolatedPoint.posx = bestPoint.posx + (point.posx - bestPoint.posx)*completion;
				interpolatedPoint.posy = bestPoint.posy + (point.posy - bestPoint.posy)*completion;
				
				Log.i("Debug", "Interpolated shift: "+(point.posx - bestPoint.posx)*completion);
				
				// iterujeme vsetky wifi, ktore sme na koncovom bode nechytali a v bestPointe hej
				search:
				for(Measurement n : bestPoint.measurements) {
					
					for(Measurement m : point.measurements)
						if(n.BSSID.equals(m.BSSID))
							continue search;
					
					float signalStrength = ((n.signalStrength + 95)*completion) - 95;
					
					// pridame
					/*
					 * !!!!!!!!!!
					 */
					if(signalStrength > -93) {
						Measurement bodicek = new Measurement(n.BSSID, n.SSID, (int)signalStrength, (int)interpolatedPoint.posx, (int)interpolatedPoint.posy);
						interpolatedPoint.measurements.add(bodicek);
					}
				}
				
				
				// iterujeme vsetky namerane wifi v koncovom bode
				for(Measurement m : point.measurements) {
					
					float signalStrength = ((m.signalStrength + 95) * completion) - 95;
					//Log.i("Debug", "MSS: "+m.signalStrength+" : "+signalStrength+" : "+m.SSID+":"+m.BSSID);
					
					/*
					 * Pozrieme ci mame danu wifi v terajsom merani
					 * matchujeme opat BSSID
					 */
					for(Measurement n : bestPoint.measurements) {
						if(n.BSSID.equals(m.BSSID)) {
							
							signalStrength = n.signalStrength + (m.signalStrength - n.signalStrength) * completion;
							break;
						}
					}
					
					//Log.i("Debug", "MSS: "+m.signalStrength+" : "+signalStrength+" (PO)");
					
					if(signalStrength > -93) {
						Measurement bodicek = new Measurement(m.BSSID, m.SSID, (int)signalStrength, (int)interpolatedPoint.posx, (int)interpolatedPoint.posy);
						interpolatedPoint.measurements.add(bodicek);
					}
				}
				
				Log.i("Debug", "=== POSX: "+interpolatedPoint.posx+" POSY: "+interpolatedPoint.posy);
				for(Measurement t : interpolatedPoint.measurements) {
					Log.i("Debug", t.SSID+" - "+t.BSSID+" - "+t.signalStrength);
				}
				
				//interpolatedPoints.add(interpolatedPoint);
				
				float deviation = calculateDeviationBetweenPoints(interpolatedPoint, currentMeasurement);
				
				Log.i("Locator", "D: "+deviation+", BD: "+bestInterpolatedDeviation);
				
				if(deviation < bestInterpolatedDeviation) {
					
					Log.i("Locator", "nasiel som lepsi bod, zhoda: "+deviation+", posun X: "+(interpolatedPoint.posx - bestInterpolatedPoint.posx)+" |Y: "+(interpolatedPoint.posy - bestInterpolatedPoint.posy));

					bestInterpolatedPoint = interpolatedPoint;
					bestInterpolatedDeviation = deviation;
					
				}
			}
		}

		if(bestInterpolatedPoint == null) // problem!!!
			return new WifiPointF(bestPoint.posx, bestPoint.posy, bestDeviation*10);
		
		return new WifiPointF(bestInterpolatedPoint.posx, bestInterpolatedPoint.posy, bestInterpolatedDeviation*10);
	}
	
	public WifiLocator(Context c) {
		mContext = c;
		mDatabase = new DatabaseHelper(mContext);
	}
	
	public void getAvailableNetworks() {
		
		WifiManager wm = (WifiManager) mContext.getSystemService(Context.WIFI_SERVICE);
		
		List<ScanResult> sr = wm.getScanResults();
		for(ScanResult result : sr) {
			Log.i("wifiscan", result.BSSID+":"+result.SSID+" - "+result.level+" ("+result.capabilities+")");
		}
	}
	
	public void saveCurrentPosition() {
		
		WifiManager wm = (WifiManager) mContext.getSystemService(Context.WIFI_SERVICE);
		
		List<ScanResult> sr = wm.getScanResults();
		for(ScanResult result : sr) {
			Log.i("DB", "Saving scan for "+result.SSID+" ("+result.BSSID+", "+result.level+", "+result.capabilities+") on position "+mX+","+mY);
			mDatabase.insertMeasurement(result.BSSID, result.SSID, result.level, mX, mY);
		}
	}
	
	public void resetData() {
		mDatabase.resetData();
	}
	
	public ArrayList<Measurement> getMeasurements() {
		Cursor c = mDatabase.getAllMeasurements();
		
		ArrayList<Measurement> results = new ArrayList<Measurement>();
		
		if(c.moveToFirst()) {
			do {
				Measurement m = new Measurement(c.getString(1), c.getString(2), c.getInt(3), c.getInt(4), c.getInt(5));
				results.add(m);
			} while(c.moveToNext());
		}
		c.close();
		
		return results;
	}
	
	public ArrayList<Measurement> getInterpolatedMeasurements() {
		return mInterpolatedMeasurements;
	}
	
	public Delaunay_Triangulation getDelaunayTriangulation() {
		return mDelaunayTriangulation;
	}
	
	public void closeDB() {
		mDatabase.close();
	}

	public void startScanOnPosition(int posx, int posy) {
		
		mX = posx;
		mY = posy;
		
		WifiManager wm = (WifiManager) mContext.getSystemService(Context.WIFI_SERVICE);
		wm.startScan();
		
	}
	
	public float distance(MeasurementPoint a, MeasurementPoint b) {
		
		double width = Math.pow(a.posx - b.posx, 2);
		double height = Math.pow(a.posy - b.posy, 2);
		
		return (float) Math.sqrt(width + height);
	}
	
	public MeasurementPoint preparePointForInterpolation(MeasurementPoint a, MeasurementPoint b, MeasurementPoint c) {
		
		search:
		for(Measurement m : b.measurements) {
			for(Measurement n : a.measurements) {
				if(m.BSSID.equals(n.BSSID))
					continue search;
			}
			
			Measurement tmp = new Measurement(m.BSSID, m.SSID, -95, m.positionX, m.positionY);
			a.measurements.add(tmp);
		}
	
		search:
		for(Measurement m : c.measurements) {
			for(Measurement n : a.measurements) {
				if(m.BSSID.equals(n.BSSID))
					continue search;
			}
			
			Measurement tmp = new Measurement(m.BSSID, m.SSID, -95, m.positionX, m.positionY);
			a.measurements.add(tmp);
		}
		
		return a;
	}
	
	public float calculateDeviationBetweenPoints(MeasurementPoint point, List<ScanResult> currentMeasurement) {
		
		float signalDifferenceSum = 0;
		int signalDifferenceItems = 0;
		
		// iterujeme vsetky wifi, ktore sme na danom mieste nechytali a teraz chytame
		search:
		for(ScanResult s : currentMeasurement) {
	
			for(Measurement m : point.measurements)
				if(s.BSSID.equals(m.BSSID))
					continue search;
			
			float signalDifference = Math.max(0, s.level + 95);
			++signalDifferenceItems;
			
			//Log.i("wifilocatordeviation", "- "+s.SSID+" (NF) ("+s.level+", "+signalDifference+")");
			signalDifferenceSum += signalDifference;
		}
		
		// iterujeme vsetky namerane wifi v danom mieste
		for(Measurement m : point.measurements) {
			
			float signalDifference = Math.max(0, m.signalStrength + 95);
			
			
			/*
			 * Pozrieme ci mame danu wifi v terajsom merani
			 * matchujeme BSSID, kedze jedna wifi ma zhodne SSID ale
			 * BSSID je unikatne pre kazde AP
			 */
			for(ScanResult s : currentMeasurement) {
				if(s.BSSID.equals(m.BSSID)) {
					signalDifference = Math.abs(s.level - m.signalStrength);
					break;
				}
			}
			
			//Log.i("wifilocatordeviation", "- "+m.SSID+" ("+m.signalStrength+", "+signalDifference+")");
			signalDifferenceSum += signalDifference;
			++signalDifferenceItems;
		}
		
		float deviation = signalDifferenceSum / signalDifferenceItems;
		
		return deviation;
	}
}
