package com.abaffydesign;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.abaffydesign.R;

import delaunay_triangulation.Delaunay_Triangulation;
import delaunay_triangulation.Triangle_dt;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Paint.Style;
import android.graphics.PointF;
import android.util.AttributeSet;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;

public class WifiMapView extends View {

	private float mX, mY, mLastX, mLastY;
	
	private Paint mPaint;
	private Paint mFontPaint;
	private Paint mRadiusPaint;
	
	private Context mContext;
	private Bitmap mMap;
	
	private WifiPointF mLocation;
	private ArrayList<Measurement> mMeasurementPoints;
	private boolean mInterpolatedMap = false;

	private float mScale;

	private Bitmap mScaledMap;

	private Paint mBgPaint;

	private Paint mGreenPaint;

	private Delaunay_Triangulation mDelaunayTriangulation;

	private Paint mInterpolatedPaint;

	private boolean mMeasurementMap = false;

	private String mostCommonWifi;
	private int highestValueWifi;
	private int lowestValueWifi;

	private boolean mHeatmap = true;

	private Paint mTriangulationPaint;
	
	

	public Delaunay_Triangulation getDelaunayTriangulation() {
		return mDelaunayTriangulation;
	}

	public void setDelaunayTriangulation(Delaunay_Triangulation delaunayTriangulation) {
		mDelaunayTriangulation = delaunayTriangulation;
	}

	public ArrayList<Measurement> getMeasurementPoints() {
		return mMeasurementPoints;
	}

	public void setMeasurementPoints(ArrayList<Measurement> measurementPoints) {
		this.mMeasurementPoints = measurementPoints;
		
		if(measurementPoints == null)
			return;
		
		System.out.println("interpolated points count: "+measurementPoints.size());
		
		HashMap<String, Integer> map = new HashMap<String, Integer>();
		lowestValueWifi = 1000;
		highestValueWifi = -1000;
		
		for(Measurement m : measurementPoints) {
			if(map.containsKey(m.BSSID)) {
				int val = map.get(m.BSSID);
				map.put(m.BSSID, val+1);
			}
			else {
				System.out.println(m.BSSID+" - "+m.SSID);
				map.put(m.BSSID, 1);
			}
		}
		
		String maxName = "";
		int maxOccurence = 0;
		for(Map.Entry<String, Integer> pair : map.entrySet()) {
			if(pair.getValue() > maxOccurence)
				maxName = pair.getKey();
		}
		
		mostCommonWifi = maxName;
		mostCommonWifi = "00:27:19:d5:50:7a";
		
		System.out.println("Most common wifi: "+mostCommonWifi);
		
		for(Measurement m : measurementPoints) {
			
			if(!m.BSSID.equals(mostCommonWifi))
				continue;
			
			lowestValueWifi = Math.min(lowestValueWifi, m.signalStrength);
			highestValueWifi = Math.max(highestValueWifi, m.signalStrength);
		}
	}

	public void setLocation(WifiPointF location) {
		mLocation = location;
		mX = -location.x + (getWidth() / 2)/mScale;
		mY = -location.y + (getHeight() / 2)/mScale;
		
		invalidate();
	}

	public WifiMapView(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);

		mContext = context;
		init();
	}

	public WifiMapView(Context context, AttributeSet attrs) {
		super(context, attrs);
		
		mContext = context;
		init();
	}

	public WifiMapView(Context context) {
		super(context);

		mContext = context;
		init();
	}

	protected void init() {
		
		mBgPaint = new Paint();
		mBgPaint.setColor(Color.WHITE);
		
		mGreenPaint = new Paint();
		mGreenPaint.setAntiAlias(true);
		mGreenPaint.setColor(Color.GREEN);
		
		mPaint = new Paint();
		mPaint.setAntiAlias(true);
		mPaint.setColor(Color.CYAN);
		
		mFontPaint = new Paint();
		mFontPaint.setAntiAlias(true);
		mFontPaint.setColor(Color.RED);
		
		mTriangulationPaint = new Paint();
		mTriangulationPaint.setAntiAlias(true);
		mTriangulationPaint.setColor(Color.argb(150, 255, 0, 0));
		
		mRadiusPaint = new Paint();
		mRadiusPaint.setColor(Color.BLUE);
		mRadiusPaint.setAntiAlias(true);
		mRadiusPaint.setStyle(Style.STROKE);
		
		mInterpolatedPaint = new Paint();
		mInterpolatedPaint.setAntiAlias(true);
		mInterpolatedPaint.setColor(Color.MAGENTA);
		
		
		mScale = 1.0f;
		mX = 0;
		mY = 0;
		
		mMap = BitmapFactory.decodeResource(mContext.getResources(), R.drawable.mapa_dom_prizemie);
		//mMap = BitmapFactory.decodeResource(mContext.getResources(), R.drawable.mapa_matfyz);
		createScaledMap();
	}
	
	protected void createScaledMap() {
		mScaledMap = Bitmap.createScaledBitmap(mMap, Math.round(mMap.getWidth() * mScale), Math.round(mMap.getHeight() * mScale), true);
	}
	
	public void zoomIn() {
		float oldScale = mScale;
		mScale = Math.min(3, mScale + 0.4f);
		createScaledMap();
		
		Log.i("AppMessages", "New viewport: "+(getWidth()/mScale)+" Old viewport: "+(getWidth()/oldScale));
		mX += (getWidth()/mScale - getWidth()/oldScale) / 2;
		mY += (getHeight()/mScale - getHeight()/oldScale) / 2;
		
		invalidate();
	}
	
	public void zoomOut() {
		float oldScale = mScale;
		mScale = Math.max(0.25f, mScale - 0.4f);
		createScaledMap();
		
		mX += (getWidth()/mScale - getWidth()/oldScale) / 2;
		mY += (getHeight()/mScale - getHeight()/oldScale) / 2;
		
		invalidate();
	}

	@Override
	protected void onDraw(Canvas canvas) {
		
		canvas.drawRect(0, 0, canvas.getWidth(), canvas.getHeight(), mBgPaint);
		
		/*
		 * Scalenuta bitmapa pre zoom
		 */
		canvas.drawBitmap(mScaledMap, mX*mScale, mY*mScale, null);
		canvas.drawText(mX+"x"+mY+" | Scale: "+mScale+" | Scaled: "+(mX*mScale)+"x"+(mY*mScale), 10, 10, mFontPaint);
		
		
		if(mMeasurementMap && mMeasurementPoints != null) {
			for(Measurement m : mMeasurementPoints) {
				
				if(!mHeatmap) {
					if(m.interpolated && mInterpolatedMap)
						canvas.drawCircle((mX+m.positionX)*mScale, (mY+m.positionY)*mScale, 7, mInterpolatedPaint);
					else
						canvas.drawCircle((mX+m.positionX)*mScale, (mY+m.positionY)*mScale, 10, mFontPaint);
				}
				else {
					if(m.BSSID.equals(mostCommonWifi)) {
						float range = (float)highestValueWifi - lowestValueWifi;
						float perc	= (highestValueWifi - m.signalStrength)/range;
						
						float[] hsv = { 250*perc, 1, 1 };
						
						Paint p = new Paint();
						p.setColor(Color.HSVToColor(250, hsv));
						canvas.drawCircle((mX+m.positionX)*mScale, (mY+m.positionY)*mScale, 7, p);
						//System.out.println("HSV: "+perc+", range: "+range+", diff: "+(highestValueWifi - m.signalStrength));
					}
				}
			}
		}
		
		if(mMeasurementMap && mDelaunayTriangulation != null) {
			
			Triangle_dt curr = null;
			Iterator<Triangle_dt> it = mDelaunayTriangulation.trianglesIterator();
			while (it.hasNext()) {
				curr = it.next();
				
				if(curr.isHalfplane())
					continue;
				
				canvas.drawLine(((int)curr.p1().x()+mX)*mScale, ((int)curr.p1().y()+mY)*mScale, ((int)curr.p2().x()+mX)*mScale, ((int)curr.p2().y()+mY)*mScale, mTriangulationPaint);
				canvas.drawLine(((int)curr.p1().x()+mX)*mScale, ((int)curr.p1().y()+mY)*mScale, ((int)curr.p3().x()+mX)*mScale, ((int)curr.p3().y()+mY)*mScale, mTriangulationPaint);
				canvas.drawLine(((int)curr.p3().x()+mX)*mScale, ((int)curr.p3().y()+mY)*mScale, ((int)curr.p2().x()+mX)*mScale, ((int)curr.p2().y()+mY)*mScale, mTriangulationPaint);
			}
			
		}
		
		
		
		
		if(mLocation != null) {
			canvas.drawCircle((mX+mLocation.x)*mScale, (mY+mLocation.y)*mScale, mLocation.radius*mScale, mRadiusPaint);
			canvas.drawCircle((mX+mLocation.x)*mScale, (mY+mLocation.y)*mScale, 2*mScale, mPaint);
		}
		
		canvas.drawCircle(canvas.getWidth()/2, canvas.getHeight()/2, 4, mGreenPaint);
		
	}

	@Override
	public boolean onTouchEvent(MotionEvent event) {
		float x = event.getX();
		float y = event.getY();

		switch (event.getAction()) {
		case MotionEvent.ACTION_MOVE:
		case MotionEvent.ACTION_UP:
			mX += (x - mLastX) / mScale;
			mY += (y - mLastY) / mScale;
			break;
		}

		mLastX = x;
		mLastY = y;

		invalidate();
		return true;
	}
	
	public PointF getMapPosition() {
		return new PointF(-mX + (getWidth()/2)/mScale, -mY + (getHeight()/2)/mScale);
	}
	
	public void switchMapMode() {
		mInterpolatedMap = !mInterpolatedMap;
		invalidate();
	}
	
	public void switchPointMode() {
		mMeasurementMap = !mMeasurementMap;
		invalidate();
	}
	
	public void switchHeatmapMode() {
		mHeatmap = !mHeatmap;
		invalidate();
	}
}
